@@ -1,5 +1,5 @@
 #include "clar_libgit2.h"
-#include "fileops.h"
+#include "futils.h"
 
 static void
 check_dirname(const char *A, const char *B)
@@ -31,15 +31,6 @@ check_basename(const char *A, const char *B)
 	git__free(base2);
 }
 
-static void
-check_topdir(const char *A, const char *B)
-{
-	const char *dir;
-
-	cl_assert((dir = git_path_topdir(A)) != NULL);
-	cl_assert_equal_s(B, dir);
-}
-
 static void
 check_joinpath(const char *path_a, const char *path_b, const char *expected_path)
 {
@@ -119,23 +110,6 @@ void test_core_path__01_basename(void)
 	check_basename(REP1024("/abc"), "abc");
 }
 
-/* get the latest component in a path */
-void test_core_path__02_topdir(void)
-{
-	check_topdir(".git/", ".git/");
-	check_topdir("/.git/", ".git/");
-	check_topdir("usr/local/.git/", ".git/");
-	check_topdir("./.git/", ".git/");
-	check_topdir("/usr/.git/", ".git/");
-	check_topdir("/", "/");
-	check_topdir("a/", "a/");
-
-	cl_assert(git_path_topdir("/usr/.git") == NULL);
-	cl_assert(git_path_topdir(".") == NULL);
-	cl_assert(git_path_topdir("") == NULL);
-	cl_assert(git_path_topdir("a") == NULL);
-}
-
 /* properly join path components */
 void test_core_path__05_joins(void)
 {
@@ -285,7 +259,7 @@ void test_core_path__08_self_join(void)
 	cl_git_pass(git_buf_joinpath(&path, path.ptr + 4, "somethinglongenoughtorealloc"));
 	cl_assert_equal_s(path.ptr, "/baz/somethinglongenoughtorealloc");
 	cl_assert(asize < path.asize);
-	
+
 	git_buf_dispose(&path);
 }
 
@@ -685,3 +659,28 @@ void test_core_path__git_path_is_file(void)
 	cl_git_pass(git_path_is_gitfile("blob", 4, GIT_PATH_GITFILE_GITATTRIBUTES, GIT_PATH_FS_HFS));
 	cl_git_fail(git_path_is_gitfile("blob", 4, 3, GIT_PATH_FS_HFS));
 }
+
+void test_core_path__validate_current_user_ownership(void)
+{
+	bool is_cur;
+
+	cl_must_pass(p_mkdir("testdir", 0777));
+	cl_git_pass(git_path_owner_is_current_user(&is_cur, "testdir"));
+	cl_assert_equal_i(is_cur, 1);
+
+	cl_git_rewritefile("testfile", "This is a test file.");
+	cl_git_pass(git_path_owner_is_current_user(&is_cur, "testfile"));
+	cl_assert_equal_i(is_cur, 1);
+
+#ifdef GIT_WIN32
+	cl_git_pass(git_path_owner_is_current_user(&is_cur, "C:\\"));
+	cl_assert_equal_i(is_cur, 0);
+
+	cl_git_fail(git_path_owner_is_current_user(&is_cur, "c:\\path\\does\\not\\exist"));
+#else
+	cl_git_pass(git_path_owner_is_current_user(&is_cur, "/"));
+	cl_assert_equal_i(is_cur, 0);
+
+	cl_git_fail(git_path_owner_is_current_user(&is_cur, "/path/does/not/exist"));
+#endif
+}
